Package: gstat
Version: 2.1-6
Title: Spatial and Spatio-Temporal Geostatistical Modelling, Prediction
        and Simulation
Authors@R: c(person(given = "Edzer", 
        family = "Pebesma", 
        role = c("aut", "cre"),
        email = "edzer.pebesma@uni-muenster.de",
        comment = c(ORCID = "0000-0001-8049-7069")),
     person("Benedikt", "Graeler", role = "aut"))
Description: Variogram modelling; simple, ordinary and universal point or block (co)kriging; spatio-temporal kriging; sequential Gaussian or indicator (co)simulation; variogram and variogram map plotting utility functions; supports sf and stars.
Depends: R (>= 2.10)
Imports: utils, stats, graphics, methods, lattice, sp (>= 0.9-72), zoo,
        sf (>= 0.7-2), sftime, spacetime (>= 1.2-8), stars, FNN
Suggests: fields, maps, mapdata, xts, raster, future, future.apply,
        RColorBrewer, geoR, ggplot2
License: GPL (>= 2.0)
URL: https://github.com/r-spatial/gstat/,
        https://r-spatial.github.io/gstat/
Encoding: UTF-8
BugReports: https://github.com/r-spatial/gstat/issues/
NeedsCompilation: yes
RoxygenNote: 6.1.1
Packaged: 2026-03-29 20:06:42 UTC; edzer
Author: Edzer Pebesma [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8049-7069>),
  Benedikt Graeler [aut]
Maintainer: Edzer Pebesma <edzer.pebesma@uni-muenster.de>
Repository: CRAN
Date/Publication: 2026-03-30 05:40:07 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 11:19:08 UTC; unix
Archs: gstat.so.dSYM
